/*
 *  rsql_plugin.h
 *
 *  Created by Marco Bambini on 1/3/08.
 *  Copyright 2005-2008 REALSoftware, Inc. All rights reserved.
 *	
 */

#ifndef _RSQL_PLUGIN_H_
#define _RSQL_PLUGIN_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifdef WIN32
#define RSQL_WINDOWS
#include <Windows.h>
#else
#ifdef __APPLE__
#define RSQL_MAC
#include <Carbon/Carbon.h>
#else
#define RSQL_LINUX
#endif
#endif
	
#ifdef WIN32
#define RSQL_API __declspec(dllexport)
#define DIRREF	HANDLE
#else
#include <dirent.h>
#define RSQL_API
#define DIRREF	DIR*
#endif

#define RSQL_API_VESION		1
#define RSQL_ANY_NUMARGS	-1

#define kFALSE				0
#define kTRUE				1
#define	kSELECT				2
#define kEXECUTE			3
#define RSQL_SEND_OK		1
#define RSQL_SEND_CURSOR	2
#define RSQL_SEND_ERROR		9
#define kCommandIDNone		0	
	
#define RSQL_UTF8           1
#define RSQL_UTF16LE        2
#define RSQL_UTF16BE        3
#define RSQL_UTF16          4    /* Use native byte order */
#define RSQL_ANY            5    /* rsql_plugin_register_function/rsql_plugin_register_aggregate only */
#define RSQL_UTF16_ALIGNED  8    /* rsql_plugin_register_collation only */

#define RSQL_LESS			-1
#define RSQL_BIG			1
#define RSQL_EQUAL			0

#define RSQL_STATIC			(void*)(0)
#define RSQL_TRANSIENT		(void*)(-1)
#define RSQL_ZEROTERM		(-1)
	
#define RSQL_INTEGER		1
#define RSQL_FLOAT			2
#define RSQL_TEXT			3
#define RSQL_BLOB			4
#define RSQL_NULL			5
	
#define SET_PLUGIN_NAME(v)			(*name=v)
#define SET_PLUGIN_VERSION(v)		(*version=v)
#define SET_PLUGIN_COPYRIGHT(v)		(*copyright=v)
#define SET_PLUGIN_DESCRIPTION(v)	(*description=v)

// privilege codes
enum {
	privilege_None				= 0,
	privilege_Admin				= 3,
	privilege_Privileges		= 4,
	privilege_Preferences		= 5,
	privilege_Databases			= 6,
	privilege_Select			= 7,
	privilege_Insert			= 8,
	privilege_Update			= 9,
	privilege_Delete			= 10,
	privilege_Pragma			= 11,
	privilege_Create			= 12,
	privilege_Drop				= 13,
	privilege_Upload			= 14,
	privilege_Download			= 15,
	privilege_Server			= 16,
	privilege_Restore			= 17,
	privilege_Backup			= 18,
	privilege_Plugin			= 19,
	
	// synthetic privileges that never appear in settings db
	privilege_Write				= 100,
	privilege_Read				= 101,
	privilege_ReadWrite			= 102
};	
	
#if defined(_MSC_VER) || defined(__BORLANDC__)
  typedef __int64 rsql_int64;
#else
  typedef long long int rsql_int64;
#endif

typedef struct rsql_context rsql_context;
typedef struct rsql_value rsql_value;
typedef struct rsql_timer rsql_timer;
typedef struct rsql_client rsql_client;
typedef struct rsql_dbref rsql_dbref;
typedef struct rsql_db rsql_db;

typedef struct PluginResolver
{
	// main functions (5)
	int		(*rsql_register_sql_function)  (void *pluginref, char *name, int num_args, int text_encoding, void *user_data, void (*xFunc)(rsql_context *ctx, int argc, rsql_value **argv));
	int		(*rsql_register_sql_aggregate) (void *pluginref, char *name, int num_args, int text_encoding, void *user_data, void (*xStep)(rsql_context *ctx, int argc, rsql_value **argv), void (*xFinal)(rsql_context *ctx));
	int		(*rsql_register_sql_collation) (void *pluginref, char *name, int text_encoding, void *user_data, int (*xCompare)(void *user_data, int len1, const void *str1, int len2, const void *str2), void(*xDestroy)(void *user_data));
	int		(*rsql_register_command) (void *pluginref, char *command, int enabled, int dbref_required, int context, int commandID, int privilegeCode, int (*xCommand)(rsql_client *client, const char **args, int commandID));
	int		(*rsql_register_protocol) (void *pluginref, int command, int bitfield, int (*xProtocol)(rsql_client *client));
	
	// timer (3)
	void	(*rsql_timer_start) (rsql_timer *t);
	void	(*rsql_timer_end) (rsql_timer *t);
	double	(*rsql_timer_value) (rsql_timer *t);
	
	// utils (16)
	int		(*rsql_file_copy) (char *dest, char *src);
	int		(*rsql_file_delete) (char *path);
	int		(*rsql_file_exists) (const char *path);
	int		(*rsql_file_rename) (const char *oldname, const char *newname);
	int		(*rsql_file_write) (char *file_path, char *buffer, int len);
	int		(*rsql_path_is_directory) (char *path);
	void	(*rsql_path_build) (char *path, char *path1, const char *path2);
	int		(*rsql_directory_create) (char *path);
	DIRREF	(*rsql_dir_init) (char *dirpath);
	char*	(*rsql_dir_read) (DIRREF dref);
	void	(*rsql_log) (char *msg);
	int		(*rsql_num_processors) (void);
	char*	(*rsql_ip_address) (void);
	char*	(*rsql_os_version) (char *buf, int len);
	char*	(*rsql_compress_buffer) (char *buffer1, int buffer1Size, char *buffer2, int buffer2Size, int *compressedSize);
	void	(*rsql_client_set_error) (rsql_client *client, int errcode, char *errmsg);

	// preferences (3)
	char*	(*rsql_pref_read) (char *key, char *domain, char *buffer, int blen);
	int		(*rsql_pref_write) (char *key, char *value, char *domain);
	int		(*rsql_pref_delete) (char *key, char *domain);
	
	// db (10)
	rsql_db*(*rsql_db_create_inmemory) (void);
	rsql_db*(*rsql_db_open) (char *name, char *key, int *flag);
	void	(*rsql_db_close) (rsql_db *db);
	int		(*rsql_db_write) (rsql_db *db, char *sql);
	int		(*rsql_db_read_table) (rsql_db *db, char *sql, char ***result, int *nrows, int *ncolumns);
	void	(*rsql_db_free_table) (char **result);
	char*	(*rsql_db_column_read) (rsql_db *db, char *sql, int colnum, char *buffer, int blen);
	void	(*rsql_db_send_cursor) (rsql_db *db, char *sql, rsql_client *client);
	int		(*rsql_db_errcode) (rsql_db *db);
	char*	(*rsql_db_errmsg) (rsql_db *db);
	
	// string (9)
	int		(*rsql_strcmpi) (const char *s1, const char *s2);
	int		(*rsql_strncmpi) (const char *s1, const char *s2, int i);
	char*	(*rsql_strdup) (const char *s);
	char*	(*rsql_strndup) (const char *s, size_t n);
	char*	(*rsql_strtrim) (char *s);
	int		(*rsql_strncasecmp) (const char *s1, const char *s2, size_t n);
	char*	(*rsql_strcasestr) (const char *s, const char *find);
	char*	(*rsql_strlowercase) (char *s);
	char*	(*rsql_struppercase) (char *s);
	
	// sqlite 3 functions (32)
	void*	(*rsql_aggregate_context) (rsql_context*, int nBytes);
	void*	(*rsql_user_data) (rsql_context*);
	void*	(*rsql_malloc) (int);
	void*	(*rsql_realloc) (void*, int);
	void	(*rsql_free) (void*);
	const void* (*rsql_value_blob) (rsql_value*);
	int		(*rsql_value_bytes) (rsql_value*);
	int		(*rsql_value_bytes16) (rsql_value*);
	double	(*rsql_value_double) (rsql_value*);
	int		(*rsql_value_int) (rsql_value*);
	rsql_int64 (*rsql_value_int64) (rsql_value*);
	const unsigned char* (*rsql_value_text) (rsql_value*);
	const void* (*rsql_value_text16) (rsql_value*);
	const void* (*rsql_value_text16le) (rsql_value*);
	const void* (*rsql_value_text16be) (rsql_value*);
	int		(*rsql_value_type) (rsql_value*);
	int		(*rsql_value_numeric_type) (rsql_value*);
	void	(*rsql_result_blob) (rsql_context*, const void*, int, void(*)(void*));
	void	(*rsql_result_double) (rsql_context*, double);
	void	(*rsql_result_error) (rsql_context*, const char*, int);
	void	(*rsql_result_error16) (rsql_context*, const void*, int);
	void	(*rsql_result_error_toobig) (rsql_context*);
	void	(*rsql_result_error_nomem) (rsql_context*);
	void	(*rsql_result_int) (rsql_context*, int);
	void	(*rsql_result_int64) (rsql_context*, rsql_int64);
	void	(*rsql_result_null) (rsql_context*);
	void	(*rsql_result_text) (rsql_context*, const char*, int, void(*)(void*));
	void	(*rsql_result_text16) (rsql_context*, const void*, int, void(*)(void*));
	void	(*rsql_result_text16le) (rsql_context*, const void*, int,void(*)(void*));
	void	(*rsql_result_text16be) (rsql_context*, const void*, int,void(*)(void*));
	void	(*rsql_result_value) (rsql_context*, rsql_value*);
	void	(*rsql_result_zeroblob) (rsql_context*, int n);	
} PluginResolver;

RSQL_API int  rsql_plugin_main (PluginResolver *resolver);
RSQL_API void rsql_plugin_init (void);
RSQL_API void rsql_plugin_register (void *pluginref);
RSQL_API void rsql_plugin_exit (void);
RSQL_API void rsql_plugin_info (char **name, char **version, char **copyright, char **description);
	
//  main functions (5)
int		rsql_register_sql_function  (void *pluginref, char *name, int num_args, int text_encoding, void *user_data, void (*xFunc)(rsql_context *ctx, int argc, rsql_value **argv));
int		rsql_register_sql_aggregate (void *pluginref, char *name, int num_args, int text_encoding, void *user_data, void (*xStep)(rsql_context *ctx, int argc, rsql_value **argv), void (*xFinal)(rsql_context *ctx));
int		rsql_register_sql_collation (void *pluginref, char *name, int text_encoding, void *user_data, int (*xCompare)(void *user_data, int len1, const void *str1, int len2, const void *str2), void(*xDestroy)(void *user_data));
int		rsql_register_command (void *pluginref, char *command, int invisible, int dbref_required, int context, int commandID, int privilegeCode, int (*xCommand)(rsql_client *client, const char **args, int commandID));
int		rsql_register_protocol (void *pluginref, int command, int bitfield, int (*xProtocol)(rsql_client *client));

// timer (3)
void	rsql_timer_start (rsql_timer *t);
void	rsql_timer_end (rsql_timer *t);
double	rsql_timer_value (rsql_timer *t);

// utils (16)
int		rsql_file_copy (char *dest, char *src);
int		rsql_file_delete (char *path);
int		rsql_file_exists (const char *path);
int		rsql_file_rename (const char *oldname, const char *newname);
int		rsql_file_write (char *file_path, char *buffer, int len);
int		rsql_path_is_directory (char *path);
void	rsql_path_build (char *path, char *path1, const char *path2);
int		rsql_directory_create (char *path);
DIRREF	rsql_dir_init (char *dirpath);
char*	rsql_dir_read (DIRREF dref);
void	rsql_log (char *msg);
int		rsql_num_processors (void);
char*	rsql_ip_address (void);
char*	rsql_os_version (char *buf, int len);
char*	rsql_compress_buffer (char *buffer1, int buffer1Size, char *buffer2, int buffer2Size, int *compressedSize);
void	rsql_client_set_error (rsql_client *client, int errcode, char *errmsg);
	
// preferences (3)
char*	rsql_pref_read (char *key, char *domain, char *buffer, int blen);
int		rsql_pref_write (char *key, char *value, char *domain);
int		rsql_pref_delete (char *key, char *domain);

// db (10)
rsql_db* rsql_db_create_inmemory (void);
rsql_db* rsql_db_open (char *name, char *key, int *flag);
void	rsql_db_close (rsql_db *db);
int		rsql_db_write (rsql_db *db, char *sql);
int		rsql_db_read_table (rsql_db *db, char *sql, char ***result, int *nrows, int *ncolumns);
void	rsql_db_free_table (char **result);
char*	rsql_db_column_read (rsql_db *db, char *sql, int colnum, char *buffer, int blen);
void	rsql_db_send_cursor (rsql_db *db, char *sql, rsql_client *client);
int		rsql_db_errcode(rsql_db *db);
char*	rsql_db_errmsg (rsql_db *db);
	
// string (9)
int		rsql_strcmpi (const char *s1, const char *s2);
int		rsql_strncmpi (const char *s1, const char *s2, int i);
char*	rsql_strdup (const char *s);
char*	rsql_strndup (const char *s, size_t n);
char*	rsql_strtrim (char *s);
int		rsql_strncasecmp (const char *s1, const char *s2, size_t n);
char*	rsql_strcasestr (const char *s, const char *find);
char*	rsql_strlowercase (char *s);
char*	rsql_struppercase (char *s);
	
// sql (32)
void	*rsql_user_data (rsql_context *ctx);
void	*rsql_aggregate_context (rsql_context*, int nbytes);
void	*rsql_malloc (int);
void	*rsql_realloc (void*, int);
void	rsql_free (void*);
const void *rsql_value_blob (rsql_value*);
int		rsql_value_bytes (rsql_value*);
int		rsql_value_bytes16 (rsql_value*);
double	rsql_value_double (rsql_value*);
int		rsql_value_int (rsql_value*);
rsql_int64 rsql_value_int64 (rsql_value*);
const unsigned char *rsql_value_text (rsql_value*);
const void *rsql_value_text16 (rsql_value*);
const void *rsql_value_text16le (rsql_value*);
const void *rsql_value_text16be (rsql_value*);
int		rsql_value_type (rsql_value*);
int		rsql_value_numeric_type (rsql_value*);
void	rsql_result_blob (rsql_context*, const void*, int, void(*)(void*));
void	rsql_result_double (rsql_context*, double);
void	rsql_result_error (rsql_context*, const char*, int);
void	rsql_result_error16 (rsql_context*, const void*, int);
void	rsql_result_error_toobig (rsql_context*);
void	rsql_result_error_nomem (rsql_context*);
void	rsql_result_int (rsql_context*, int);
void	rsql_result_int64 (rsql_context*, rsql_int64);
void	rsql_result_null (rsql_context*);
void	rsql_result_text (rsql_context*, const char*, int, void(*)(void*));
void	rsql_result_text16 (rsql_context*, const void*, int, void(*)(void*));
void	rsql_result_text16le (rsql_context*, const void*, int,void(*)(void*));
void	rsql_result_text16be (rsql_context*, const void*, int,void(*)(void*));
void	rsql_result_value (rsql_context*, rsql_value*);
void	rsql_result_zeroblob (rsql_context*, int n);
	
#ifdef __cplusplus
}
#endif

#endif
